--
-- $Id: ParameterizedText.hs,v 1.4 2005/12/16 16:13:47 aamine Exp $
--
-- Copyright (c) 2005 Minero Aoki
--
-- This program is free software.
-- You can distribute/modify this program under the terms of
-- the GNU LGPL, Lesser General Public License version 2.1.
-- For details of the GNU LGPL, see the file "COPYING".
--

module ParameterizedText
    (ParameterizedText, PTItem(..),
     resolve, texts, parameters, isText, isParam) where

import Data.Maybe

type ParameterizedText a = [PTItem a]
data PTItem a = Text String | Param a

resolve :: (a -> String) -> ParameterizedText a -> String
resolve _ [] = ""
resolve f (x:xs) = textify x ++ resolve f xs
  where
    textify (Text x)  = x
    textify (Param x) = f x

texts :: ParameterizedText a -> [String]
texts = mapMaybe (getText)

parameters :: ParameterizedText a -> [a]
parameters = mapMaybe (getParam)

getText (Text s) = Just s
getText (Param _) = Nothing

getParam (Param x) = Just x
getParam (Text _) = Nothing

isText :: PTItem a -> Bool
isText (Text _) = True
isText (Param _) = False

isParam :: PTItem a -> Bool
isParam = not . isText
